@@ -1,22 +1,4 @@
-/* liblxcapi
- *
- * Copyright © 2019 Christian Brauner <christian.brauner@ubuntu.com>.
- * Copyright © 2019 Canonical Ltd.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
-
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
-
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -24,6 +6,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <linux/magic.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/sendfile.h>
@@ -31,15 +14,66 @@
 
 #include "config.h"
 #include "file_utils.h"
+#include "log.h"
 #include "macro.h"
 #include "memory_utils.h"
 #include "string_utils.h"
 #include "utils.h"
 
+int lxc_open_dirfd(const char *dir)
+{
+	return open(dir, O_DIRECTORY | O_RDONLY | O_CLOEXEC | O_NOFOLLOW);
+}
+
+int lxc_readat(int dirfd, const char *filename, void *buf, size_t count)
+{
+	__do_close int fd = -EBADF;
+	ssize_t ret;
+
+	fd = openat(dirfd, filename, O_RDONLY | O_CLOEXEC);
+	if (fd < 0)
+		return -1;
+
+	ret = lxc_read_nointr(fd, buf, count);
+	if (ret < 0 || (size_t)ret != count)
+		return -1;
+
+	return 0;
+}
+
+int lxc_writeat(int dirfd, const char *filename, const void *buf, size_t count)
+{
+	__do_close int fd = -EBADF;
+	ssize_t ret;
+
+	fd = openat(dirfd, filename,
+		    O_WRONLY | O_CLOEXEC | O_NOCTTY | O_NOFOLLOW);
+	if (fd < 0)
+		return -1;
+
+	ret = lxc_write_nointr(fd, buf, count);
+	if (ret < 0 || (size_t)ret != count)
+		return -1;
+
+	return 0;
+}
+
+int lxc_write_openat(const char *dir, const char *filename, const void *buf,
+		     size_t count)
+{
+	__do_close int dirfd = -EBADF;
+
+	dirfd = open(dir, O_DIRECTORY | O_RDONLY | O_CLOEXEC | O_NOCTTY | O_NOFOLLOW);
+	if (dirfd < 0)
+		return -1;
+
+	return lxc_writeat(dirfd, filename, buf, count);
+}
+
 int lxc_write_to_file(const char *filename, const void *buf, size_t count,
 		      bool add_newline, mode_t mode)
 {
-	int fd, saved_errno;
+	__do_close int fd = -EBADF;
 	ssize_t ret;
 
 	fd = open(filename, O_WRONLY | O_TRUNC | O_CREAT | O_CLOEXEC, mode);
@@ -48,30 +82,23 @@ int lxc_write_to_file(const char *filename, const void *buf, size_t count,
 
 	ret = lxc_write_nointr(fd, buf, count);
 	if (ret < 0)
-		goto out_error;
+		return -1;
 
 	if ((size_t)ret != count)
-		goto out_error;
+		return -1;
 
 	if (add_newline) {
 		ret = lxc_write_nointr(fd, "\n", 1);
 		if (ret != 1)
-			goto out_error;
+			return -1;
 	}
 
-	close(fd);
 	return 0;
-
-out_error:
-	saved_errno = errno;
-	close(fd);
-	errno = saved_errno;
-	return -1;
 }
 
 int lxc_read_from_file(const char *filename, void *buf, size_t count)
 {
-	int fd = -1, saved_errno;
+	__do_close int fd = -EBADF;
 	ssize_t ret;
 
 	fd = open(filename, O_RDONLY | O_CLOEXEC);
@@ -92,19 +119,27 @@ int lxc_read_from_file(const char *filename, void *buf, size_t count)
 		ret = lxc_read_nointr(fd, buf, count);
 	}
 
-	saved_errno = errno;
-	close(fd);
-	errno = saved_errno;
 	return ret;
 }
 
 ssize_t lxc_write_nointr(int fd, const void *buf, size_t count)
 {
 	ssize_t ret;
-again:
-	ret = write(fd, buf, count);
-	if (ret < 0 && errno == EINTR)
-		goto again;
+
+	do {
+		ret = write(fd, buf, count);
+	} while (ret < 0 && errno == EINTR);
+
+	return ret;
+}
+
+ssize_t lxc_pwrite_nointr(int fd, const void *buf, size_t count, off_t offset)
+{
+	ssize_t ret;
+
+	do {
+		ret = pwrite(fd, buf, count, offset);
+	} while (ret < 0 && errno == EINTR);
 
 	return ret;
 }
@@ -112,10 +147,10 @@ again:
 ssize_t lxc_send_nointr(int sockfd, void *buf, size_t len, int flags)
 {
 	ssize_t ret;
-again:
-	ret = send(sockfd, buf, len, flags);
-	if (ret < 0 && errno == EINTR)
-		goto again;
+
+	do {
+		ret = send(sockfd, buf, len, flags);
+	} while (ret < 0 && errno == EINTR);
 
 	return ret;
 }
@@ -123,10 +158,10 @@ again:
 ssize_t lxc_read_nointr(int fd, void *buf, size_t count)
 {
 	ssize_t ret;
-again:
-	ret = read(fd, buf, count);
-	if (ret < 0 && errno == EINTR)
-		goto again;
+
+	do {
+		ret = read(fd, buf, count);
+	} while (ret < 0 && errno == EINTR);
 
 	return ret;
 }
@@ -134,31 +169,58 @@ again:
 ssize_t lxc_recv_nointr(int sockfd, void *buf, size_t len, int flags)
 {
 	ssize_t ret;
-again:
-	ret = recv(sockfd, buf, len, flags);
-	if (ret < 0 && errno == EINTR)
-		goto again;
+
+	do {
+		ret = recv(sockfd, buf, len, flags);
+	} while (ret < 0 && errno == EINTR);
 
 	return ret;
 }
 
-ssize_t lxc_read_nointr_expect(int fd, void *buf, size_t count, const void *expected_buf)
+ssize_t lxc_recvmsg_nointr_iov(int sockfd, struct iovec *iov, size_t iovlen,
+			       int flags)
+{
+	ssize_t ret;
+	struct msghdr msg = {
+		.msg_iov = iov,
+		.msg_iovlen = iovlen,
+	};
+
+	do {
+		ret = recvmsg(sockfd, &msg, flags);
+	} while (ret < 0 && errno == EINTR);
+
+	return ret;
+}
+
+ssize_t lxc_read_nointr_expect(int fd, void *buf, size_t count,
+			       const void *expected_buf)
 {
 	ssize_t ret;
 
 	ret = lxc_read_nointr(fd, buf, count);
-	if (ret <= 0)
+	if (ret < 0)
 		return ret;
 
 	if ((size_t)ret != count)
 		return -1;
 
-	if (expected_buf && memcmp(buf, expected_buf, count) != 0) {
-		errno = EINVAL;
-		return -1;
-	}
+	if (expected_buf && memcmp(buf, expected_buf, count) != 0)
+		return ret_set_errno(-1, EINVAL);
 
-	return ret;
+	return 0;
+}
+
+ssize_t lxc_read_file_expect(const char *path, void *buf, size_t count,
+			     const void *expected_buf)
+{
+	__do_close int fd = -EBADF;
+
+	fd = open(path, O_RDONLY | O_CLOEXEC);
+	if (fd < 0)
+		return -1;
+
+	return lxc_read_nointr_expect(fd, buf, count, expected_buf);
 }
 
 bool file_exists(const char *f)
@@ -170,24 +232,23 @@ bool file_exists(const char *f)
 
 int print_to_file(const char *file, const char *content)
 {
-	FILE *f;
+	__do_fclose FILE *f = NULL;
 	int ret = 0;
 
-	f = fopen(file, "w");
+	f = fopen(file, "we");
 	if (!f)
 		return -1;
 
 	if (fprintf(f, "%s", content) != strlen(content))
 		ret = -1;
 
-	fclose(f);
 	return ret;
 }
 
 int is_dir(const char *path)
 {
-	struct stat statbuf;
 	int ret;
+	struct stat statbuf;
 
 	ret = stat(path, &statbuf);
 	if (ret == 0 && S_ISDIR(statbuf.st_mode))
@@ -201,8 +262,8 @@ int is_dir(const char *path)
  */
 int lxc_count_file_lines(const char *fn)
 {
-	FILE *f;
-	char *line = NULL;
+	__do_free char *line = NULL;
+	__do_fclose FILE *f = NULL;
 	size_t sz = 0;
 	int n = 0;
 
@@ -210,18 +271,15 @@ int lxc_count_file_lines(const char *fn)
 	if (!f)
 		return -1;
 
-	while (getline(&line, &sz, f) != -1) {
+	while (getline(&line, &sz, f) != -1)
 		n++;
-	}
 
-	free(line);
-	fclose(f);
 	return n;
 }
 
 int lxc_make_tmpfile(char *template, bool rm)
 {
-	__do_close_prot_errno int fd = -EBADF;
+	__do_close int fd = -EBADF;
 	int ret;
 	mode_t msk;
 
@@ -275,11 +333,9 @@ bool fhas_fs_type(int fd, fs_type_magic magic_val)
 
 FILE *fopen_cloexec(const char *path, const char *mode)
 {
-	int open_mode = 0;
-	int step = 0;
-	int fd;
-	int saved_errno = 0;
-	FILE *ret;
+	__do_close int fd = -EBADF;
+	int open_mode = 0, step = 0;
+	FILE *f;
 
 	if (!strncmp(mode, "r+", 2)) {
 		open_mode = O_RDWR;
@@ -303,75 +359,28 @@ FILE *fopen_cloexec(const char *path, const char *mode)
 	for (; mode[step]; step++)
 		if (mode[step] == 'x')
 			open_mode |= O_EXCL;
-	open_mode |= O_CLOEXEC;
 
-	fd = open(path, open_mode, 0660);
+	fd = open(path, open_mode | O_CLOEXEC, 0660);
 	if (fd < 0)
 		return NULL;
 
-	ret = fdopen(fd, mode);
-	saved_errno = errno;
-	if (!ret)
-		close(fd);
-	errno = saved_errno;
-	return ret;
+	f = fdopen(fd, mode);
+	if (f)
+		move_fd(fd);
+	return f;
 }
 
 ssize_t lxc_sendfile_nointr(int out_fd, int in_fd, off_t *offset, size_t count)
 {
 	ssize_t ret;
 
-again:
-	ret = sendfile(out_fd, in_fd, offset, count);
-	if (ret < 0) {
-		if (errno == EINTR)
-			goto again;
-
-		return -1;
-	}
+	do {
+		ret = sendfile(out_fd, in_fd, offset, count);
+	} while (ret < 0 && errno == EINTR);
 
 	return ret;
 }
 
-char *file_to_buf(char *path, size_t *length)
-{
-	int fd;
-	char buf[PATH_MAX];
-	char *copy = NULL;
-
-	if (!length)
-		return NULL;
-
-	fd = open(path, O_RDONLY | O_CLOEXEC);
-	if (fd < 0)
-		return NULL;
-
-	*length = 0;
-	for (;;) {
-		int n;
-		char *old = copy;
-
-		n = lxc_read_nointr(fd, buf, sizeof(buf));
-		if (n < 0)
-			goto on_error;
-		if (!n)
-			break;
-
-		copy = must_realloc(old, (*length + n) * sizeof(*old));
-		memcpy(copy + *length, buf, n);
-		*length += n;
-	}
-
-	close(fd);
-	return copy;
-
-on_error:
-	close(fd);
-	free(copy);
-
-	return NULL;
-}
-
 int fd_to_fd(int from, int to)
 {
 	for (;;) {
@@ -401,3 +410,105 @@ int fd_to_fd(int from, int to)
 
 	return 0;
 }
+
+int fd_to_buf(int fd, char **buf, size_t *length)
+{
+	__do_free char *copy = NULL;
+
+	if (!length)
+		return 0;
+
+	*length = 0;
+	for (;;) {
+		ssize_t bytes_read;
+		char chunk[4096];
+		char *old = copy;
+
+		bytes_read = lxc_read_nointr(fd, chunk, sizeof(chunk));
+		if (bytes_read < 0)
+			return 0;
+
+		if (!bytes_read)
+			break;
+
+		copy = must_realloc(old, (*length + bytes_read) * sizeof(*old));
+		memcpy(copy + *length, chunk, bytes_read);
+		*length += bytes_read;
+	}
+
+	*buf = move_ptr(copy);
+	return 0;
+}
+
+char *file_to_buf(const char *path, size_t *length)
+{
+	__do_close int fd = -EBADF;
+	char *buf = NULL;
+
+	if (!length)
+		return NULL;
+
+	fd = open(path, O_RDONLY | O_CLOEXEC);
+	if (fd < 0)
+		return NULL;
+
+	if (fd_to_buf(fd, &buf, length) < 0)
+		return NULL;
+
+	return buf;
+}
+
+FILE *fopen_cached(const char *path, const char *mode, void **caller_freed_buffer)
+{
+#ifdef HAVE_FMEMOPEN
+	__do_free char *buf = NULL;
+	size_t len = 0;
+	FILE *f;
+
+	buf = file_to_buf(path, &len);
+	if (!buf)
+		return NULL;
+
+	f = fmemopen(buf, len, mode);
+	if (!f)
+		return NULL;
+	*caller_freed_buffer = move_ptr(buf);
+	return f;
+#else
+	return fopen(path, mode);
+#endif
+}
+
+FILE *fdopen_cached(int fd, const char *mode, void **caller_freed_buffer)
+{
+	FILE *f;
+#ifdef HAVE_FMEMOPEN
+	__do_free char *buf = NULL;
+	size_t len = 0;
+
+	if (fd_to_buf(fd, &buf, &len) < 0)
+		return NULL;
+
+	f = fmemopen(buf, len, mode);
+	if (!f)
+		return NULL;
+
+	*caller_freed_buffer = move_ptr(buf);
+
+#else
+
+	__do_close int dupfd = -EBADF;
+
+	dupfd = dup(fd);
+	if (dupfd < 0)
+		return NULL;
+
+	f = fdopen(dupfd, "re");
+	if (!f)
+		return NULL;
+
+	/* Transfer ownership of fd. */
+	move_fd(dupfd);
+#endif
+	return f;
+}
