@@ -1,24 +1,8 @@
-/* liblxcapi
- *
- * Copyright © 2012 Serge Hallyn <serge.hallyn@ubuntu.com>.
- * Copyright © 2012 Canonical Ltd.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public
- *  License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
-
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- */
-
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #include <errno.h>
 #include <fcntl.h>
 #include <malloc.h>
@@ -30,9 +14,10 @@
 
 #include <lxc/lxccontainer.h>
 
+#include "config.h"
+#include "log.h"
 #include "lxclock.h"
 #include "utils.h"
-#include "log.h"
 
 #ifdef MUTEX_DEBUGGING
 #include <execinfo.h>
@@ -106,7 +91,7 @@ static char *lxclock_name(const char *p, const char *n)
 	 */
 
 	/* length of "/lxc/lock/" + $lxcpath + "/" + "." + $lxcname + '\0' */
-	len = (sizeof("/lxc/lock/") - 1) + strlen(n) + strlen(p) + 3;
+	len = STRLITERALLEN("/lxc/lock/") + strlen(n) + strlen(p) + 3;
 
 	rundir = get_rundir();
 	if (!rundir)
@@ -184,6 +169,8 @@ struct lxc_lock *lxc_newlock(const char *lxcpath, const char *name)
 	l->type = LXC_LOCK_FLOCK;
 	l->u.f.fname = lxclock_name(lxcpath, name);
 	if (!l->u.f.fname) {
+		if (!name)
+			free(l->u.sem);
 		free(l);
 		l = NULL;
 		goto on_error;
