@@ -1,25 +1,4 @@
-/*
- * lxc: linux Container library
- *
- * (C) Copyright IBM Corp. 2007, 2008
- *
- * Authors:
- * Daniel Lezcano <daniel.lezcano at free.fr>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -55,7 +34,7 @@ int lxc_mainloop(struct lxc_epoll_descr *descr, int timeout_ms)
 			if (errno == EINTR)
 				continue;
 
-			return -1;
+			return -errno;
 		}
 
 		for (i = 0; i < nfds; i++) {
@@ -66,6 +45,8 @@ int lxc_mainloop(struct lxc_epoll_descr *descr, int timeout_ms)
 			 */
 			ret = handler->callback(handler->fd, events[i].events,
 						handler->data, descr);
+			if (ret == LXC_MAINLOOP_ERROR)
+				return -1;
 			if (ret == LXC_MAINLOOP_CLOSE)
 				return 0;
 		}
@@ -81,9 +62,12 @@ int lxc_mainloop(struct lxc_epoll_descr *descr, int timeout_ms)
 int lxc_mainloop_add_handler(struct lxc_epoll_descr *descr, int fd,
 			     lxc_mainloop_callback_t callback, void *data)
 {
+	__do_free struct mainloop_handler *handler = NULL;
+	__do_free struct lxc_list *item = NULL;
 	struct epoll_event ev;
-	struct mainloop_handler *handler;
-	struct lxc_list *item;
+
+	if (fd < 0)
+		return -1;
 
 	handler = malloc(sizeof(*handler));
 	if (!handler)
@@ -97,19 +81,15 @@ int lxc_mainloop_add_handler(struct lxc_epoll_descr *descr, int fd,
 	ev.data.ptr = handler;
 
 	if (epoll_ctl(descr->epfd, EPOLL_CTL_ADD, fd, &ev) < 0)
-		goto out_free_handler;
+		return -errno;
 
 	item = malloc(sizeof(*item));
 	if (!item)
-		goto out_free_handler;
+		return ret_errno(ENOMEM);
 
-	item->elem = handler;
-	lxc_list_add(&descr->handlers, item);
+	item->elem = move_ptr(handler);
+	lxc_list_add(&descr->handlers, move_ptr(item));
 	return 0;
-
-out_free_handler:
-	free(handler);
-	return -1;
 }
 
 int lxc_mainloop_del_handler(struct lxc_epoll_descr *descr, int fd)
@@ -123,7 +103,7 @@ int lxc_mainloop_del_handler(struct lxc_epoll_descr *descr, int fd)
 		if (handler->fd == fd) {
 			/* found */
 			if (epoll_ctl(descr->epfd, EPOLL_CTL_DEL, fd, NULL))
-				return -1;
+				return -errno;
 
 			lxc_list_del(iterator);
 			free(iterator->elem);
@@ -132,21 +112,20 @@ int lxc_mainloop_del_handler(struct lxc_epoll_descr *descr, int fd)
 		}
 	}
 
-	return -1;
+	return ret_errno(EINVAL);
 }
 
 int lxc_mainloop_open(struct lxc_epoll_descr *descr)
 {
-	/* hint value passed to epoll create */
 	descr->epfd = epoll_create1(EPOLL_CLOEXEC);
 	if (descr->epfd < 0)
-		return -1;
+		return -errno;
 
 	lxc_list_init(&descr->handlers);
 	return 0;
 }
 
-int lxc_mainloop_close(struct lxc_epoll_descr *descr)
+void lxc_mainloop_close(struct lxc_epoll_descr *descr)
 {
 	struct lxc_list *iterator, *next;
 
@@ -160,8 +139,5 @@ int lxc_mainloop_close(struct lxc_epoll_descr *descr)
 		iterator = next;
 	}
 
-	if (descr->epfd >= 0)
-		return close(descr->epfd);
-
-	return 0;
+	close_prot_errno_disarm(descr->epfd);
 }
