@@ -1,25 +1,4 @@
-/*
- * lxc: linux Container library
- *
- * (C) Copyright IBM Corp. 2007, 2008
- *
- * Authors:
- * Daniel Lezcano <daniel.lezcano at free.fr>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -84,6 +63,8 @@ Options :\n\
 	.log_priority = "ERROR",
 	.log_file     = "none",
 	.daemonize    = 0,
+	.uid          = LXC_INVALID_UID,
+	.gid          = LXC_INVALID_GID,
 };
 
 static int my_parser(struct lxc_arguments *args, int c, char *arg)
@@ -211,7 +192,7 @@ int main(int argc, char *argv[])
 	if (!bret)
 		goto out;
 
-	if (my_args.uid) {
+	if (my_args.uid != LXC_INVALID_UID) {
 		char buf[256];
 
 		ret = snprintf(buf, 256, "%d", my_args.uid);
@@ -223,7 +204,7 @@ int main(int argc, char *argv[])
 			goto out;
 	}
 
-	if (my_args.gid) {
+	if (my_args.gid != LXC_INVALID_GID) {
 		char buf[256];
 
 		ret = snprintf(buf, 256, "%d", my_args.gid);
